@@ -144,6 +144,13 @@ public:
         return getData("dup_id");
     }
 
+    void setVersionFixedIn(const QString & dupID) {
+        setData("cf_versionfixedin", dupID);
+    }
+    QString versionFixedIn() const {
+        return getData("cf_versionfixedin");
+    }
+    
     void setValid(bool valid) {
         m_isValid = valid;
     }
@@ -224,6 +231,8 @@ public:
     
     void attachTextToReport(const QString &, const QString &, const QString &, uint, const QString &);
 
+    void addCommentToReport(const QString &, int, bool);
+
     void addMeToCC(int);
 
     void checkVersionsForProduct(const QString &);
@@ -243,6 +252,8 @@ private Q_SLOTS:
     void searchBugsJobFinished(KJob*);
     void sendReportJobFinished(KJob*);
     void attachToReportJobFinished(KJob*);
+    void addCommentSubJobFinished(KJob*);
+    void addCommentJobFinished(KJob*);
     void addMeToCCSubJobFinished(KJob*);
     void addMeToCCJobFinished(KJob*);
     void checkVersionJobFinished(KJob*);
@@ -255,6 +266,7 @@ Q_SIGNALS:
     void searchFinished(const BugMapList &);
     void reportSent(int);
     void attachToReportSent(int, int);
+    void addCommentFinished(int);
     void addMeToCCFinished(int);
     void checkVersionsForProductFinished(const QStringList);
 
@@ -265,11 +277,15 @@ Q_SIGNALS:
     void sendReportError(const QString &);
     void sendReportErrorInvalidValues(); //To use default values
     void attachToReportError(const QString &);
+    void addCommentError(const QString &);
     void addMeToCCError(const QString &);
     void checkVersionsForProductError();
 
 private:
     /* Private helper methods */
+    QString getToken(const QString &);
+    QString getErrorMessage(const QString &, bool fallBackMessage = true);
+
     void attachToReport(const QByteArray &, const QByteArray &);
     QByteArray generatePostDataForReport(BugReport) const;
 
