@@ -21,24 +21,19 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include "localemon.h"
+
 #include <kdebug.h>
 #include <QCheckBox>
 #include <QComboBox>
-#include <QLabel>
-#include <QLineEdit>
 
-#include <QLayout>
-
-#include <QRegExp>
-
-#include <KNumInput>
-#include <KDialog>
 #include <KConfig>
 #include <KConfigGroup>
 #include <KStandardDirs>
+#include <KIntNumInput>
+#include "kcurrencycode.h"
 
 #include "toplevel.h"
-#include "localemon.h"
 #include "localemon.moc"
 
 KLocaleConfigMoney::KLocaleConfigMoney(KLocale *locale,
@@ -49,10 +44,11 @@ KLocaleConfigMoney::KLocaleConfigMoney(KLocale *locale,
   setupUi(this);
 
   // Money
-  m_labMonCurSym->setObjectName( I18N_NOOP("Currency symbol:") );
+  m_labelCurrencyCode->setObjectName( I18N_NOOP("Currency:") );
+  m_labelCurrencySymbol->setObjectName( I18N_NOOP("Currency symbol:") );
   m_labMonDecSym->setObjectName( I18N_NOOP("Decimal symbol:") );
   m_labMonThoSep->setObjectName( I18N_NOOP("Thousands separator:") );
-  m_labMonFraDig->setObjectName( I18N_NOOP("Fract digits:") );
+  m_labelMonetaryDecimalPlaces->setObjectName( I18N_NOOP("Decimal places:") );
   m_positiveGB->setObjectName( I18N_NOOP("Positive") );
   m_chMonPosPreCurSym->setObjectName(I18N_NOOP("Prefix currency symbol"));
   m_labMonPosMonSignPos->setObjectName( I18N_NOOP("Sign position:") );
@@ -61,8 +57,11 @@ KLocaleConfigMoney::KLocaleConfigMoney(KLocale *locale,
   m_labMonNegMonSignPos->setObjectName( I18N_NOOP("Sign position:") );
   m_labMonDigSet->setObjectName( I18N_NOOP("Digit set:") );
 
-  connect( m_edMonCurSym, SIGNAL( textChanged(const QString &) ),
-           SLOT( slotMonCurSymChanged(const QString &) ) );
+  connect( m_comboCurrencyCode, SIGNAL( activated(int) ),
+           SLOT( slotCurrencyCodeChanged(int) ) );
+
+  connect( m_comboCurrencySymbol, SIGNAL( activated(int) ),
+           SLOT( slotCurrencySymbolChanged(int) ) );
 
   connect( m_edMonDecSym, SIGNAL( textChanged(const QString &) ),
            SLOT( slotMonDecSymChanged(const QString &) ) );
@@ -70,8 +69,8 @@ KLocaleConfigMoney::KLocaleConfigMoney(KLocale *locale,
   connect( m_edMonThoSep, SIGNAL( textChanged(const QString &) ),
            SLOT( slotMonThoSepChanged(const QString &) ) );
 
-  connect( m_inMonFraDig, SIGNAL( valueChanged(int) ),
-           SLOT( slotMonFraDigChanged(int) ) );
+  connect( m_intMonetaryDecimalPlaces, SIGNAL( valueChanged(int) ),
+           SLOT( slotMonetaryDecimalPlacesChanged(int) ) );
 
   connect( m_chMonPosPreCurSym, SIGNAL( clicked() ),
            SLOT( slotMonPosPreCurSymChanged() ) );
@@ -87,9 +86,6 @@ KLocaleConfigMoney::KLocaleConfigMoney(KLocale *locale,
 
   connect( m_cmbMonDigSet, SIGNAL( activated(int) ),
            SLOT( slotMonDigSetChanged(int) ) );
-
-  m_inMonFraDig->setRange(0, 10, 1);
-  m_inMonFraDig->setSliderEnabled(false);
 }
 
 KLocaleConfigMoney::~KLocaleConfigMoney()
@@ -109,11 +105,17 @@ void KLocaleConfigMoney::save()
   int i;
   bool b;
 
-  str = entGrp.readEntry("CurrencySymbol", QString::fromLatin1("$"));
+  QString currencyCode = entGrp.readEntry("CurrencyCode", m_locale->defaultCurrencyCode());
+  group.deleteEntry("CurrencyCode", KConfig::Persistent | KConfig::Global);
+  if (currencyCode != m_locale->currencyCode()) {
+    group.writeEntry("CurrencyCode", m_locale->currencyCode(), KConfig::Persistent|KConfig::Global);
+  }
+
+  QString currencySymbol = entGrp.readEntry("CurrencySymbol", m_locale->currency()->defaultSymbol());
   group.deleteEntry("CurrencySymbol", KConfig::Persistent | KConfig::Global);
-  if (str != m_locale->currencySymbol())
-    group.writeEntry("CurrencySymbol",
-                       m_locale->currencySymbol(), KConfig::Persistent|KConfig::Global);
+  if (currencySymbol != m_locale->currencySymbol()) {
+    group.writeEntry("CurrencySymbol", m_locale->currencySymbol(), KConfig::Persistent|KConfig::Global);
+  }
 
   str = entGrp.readEntry("MonetaryDecimalSymbol", QString::fromLatin1("."));
   group.deleteEntry("MonetaryDecimalSymbol", KConfig::Persistent | KConfig::Global);
@@ -130,10 +132,11 @@ void KLocaleConfigMoney::save()
                        .arg(m_locale->monetaryThousandsSeparator()),
                        KConfig::Persistent|KConfig::Global);
 
-  i = entGrp.readEntry("FracDigits", 2);
-  group.deleteEntry("FracDigits", KConfig::Persistent | KConfig::Global);
-  if (i != m_locale->fracDigits())
-    group.writeEntry("FracDigits", m_locale->fracDigits(), KConfig::Persistent|KConfig::Global);
+  int monetaryDecimalPlaces = entGrp.readEntry("MonetaryDecimalPlaces", m_locale->currency()->decimalPlaces());
+  group.deleteEntry("MonetaryDecimalPlaces", KConfig::Persistent | KConfig::Global);
+  if (monetaryDecimalPlaces != m_locale->monetaryDecimalPlaces()) {
+    group.writeEntry("MonetaryDecimalPlaces", m_locale->monetaryDecimalPlaces(), KConfig::Persistent|KConfig::Global);
+  }
 
   b = entGrp.readEntry("PositivePrefixCurrencySymbol", true);
   group.deleteEntry("PositivePrefixCurrencySymbol", KConfig::Persistent | KConfig::Global);
@@ -175,10 +178,23 @@ void KLocaleConfigMoney::save()
 
 void KLocaleConfigMoney::slotLocaleChanged()
 {
-  m_edMonCurSym->setText( m_locale->currencySymbol() );
+  m_comboCurrencyCode->setCurrentIndex( m_comboCurrencyCode->findData( QVariant( m_locale->currencyCode() ) ) );
+
+  //Create the list of Currency Symbols for the selected Currency Code
+  m_comboCurrencySymbol->clear();
+  QStringList currencySymbolList = m_locale->currency()->symbolList();
+  foreach ( const QString &currencySymbol, currencySymbolList ) {
+      m_comboCurrencySymbol->addItem( currencySymbol, QVariant( currencySymbol ) );
+  }
+  int i = m_comboCurrencySymbol->findData( QVariant( m_locale->currencySymbol() ) );
+  if ( i < 0 ) {
+      i = m_comboCurrencySymbol->findData( QVariant( m_locale->currency()->defaultSymbol() ) );
+  }
+  m_comboCurrencySymbol->setCurrentIndex( i );
+
   m_edMonDecSym->setText( m_locale->monetaryDecimalSymbol() );
   m_edMonThoSep->setText( m_locale->monetaryThousandsSeparator() );
-  m_inMonFraDig->setValue( m_locale->fracDigits() );
+  m_intMonetaryDecimalPlaces->setValue( m_locale->monetaryDecimalPlaces() );
 
   m_chMonPosPreCurSym->setChecked( m_locale->positivePrefixCurrencySymbol() );
   m_chMonNegPreCurSym->setChecked( m_locale->negativePrefixCurrencySymbol() );
@@ -188,10 +204,26 @@ void KLocaleConfigMoney::slotLocaleChanged()
   m_cmbMonDigSet->setCurrentIndex( m_locale->monetaryDigitSet() );
 }
 
-void KLocaleConfigMoney::slotMonCurSymChanged(const QString &t)
+void KLocaleConfigMoney::slotCurrencyCodeChanged( int i )
 {
-  m_locale->setCurrencySymbol(t);
-  emit localeChanged();
+    m_locale->setCurrencyCode( m_comboCurrencyCode->itemData( i ).toString() );
+
+    //Create the list of Currency Symbols for the selected Currency Code
+    m_comboCurrencySymbol->clear();
+    QStringList currencySymbolList = m_locale->currency()->symbolList();
+    foreach ( const QString &currencySymbol, currencySymbolList ) {
+        m_comboCurrencySymbol->addItem( currencySymbol, QVariant( currencySymbol ) );
+    }
+    m_comboCurrencySymbol->setCurrentIndex( m_comboCurrencySymbol->findData( QVariant( m_locale->currency()->defaultSymbol() ) ) );
+    slotCurrencySymbolChanged( m_comboCurrencySymbol->currentIndex() );
+
+    emit localeChanged();
+}
+
+void KLocaleConfigMoney::slotCurrencySymbolChanged( int i )
+{
+    m_locale->setCurrencySymbol( m_comboCurrencySymbol->itemData( i ).toString() );
+    emit localeChanged();
 }
 
 void KLocaleConfigMoney::slotMonDecSymChanged(const QString &t)
@@ -206,10 +238,10 @@ void KLocaleConfigMoney::slotMonThoSepChanged(const QString &t)
   emit localeChanged();
 }
 
-void KLocaleConfigMoney::slotMonFraDigChanged(int value)
+void KLocaleConfigMoney::slotMonetaryDecimalPlacesChanged(int value)
 {
-  m_locale->setFracDigits(value);
-  emit localeChanged();
+    m_locale->setMonetaryDecimalPlaces(value);
+    emit localeChanged();
 }
 
 void KLocaleConfigMoney::slotMonPosPreCurSymChanged()
@@ -244,6 +276,38 @@ void KLocaleConfigMoney::slotMonDigSetChanged(int i)
 
 void KLocaleConfigMoney::slotTranslate()
 {
+  //Create the list of Currency Codes to choose from.
+  //Visible text will be localised name plus unlocalised code.
+  m_comboCurrencyCode->clear();
+  //First put all the preferred currencies first in order of priority
+  QStringList currencyCodeList = m_locale->currencyCodeList();
+  foreach ( const QString &currencyCode, currencyCodeList ) {
+      QString text = i18nc( "@item currency name and currency code", "%1 (%2)",
+                            m_locale->currency()->currencyCodeToName( currencyCode ), currencyCode );
+    m_comboCurrencyCode->addItem( text, QVariant( currencyCode ) );
+  }
+  //Next put all currencies available in alphabetical name order
+  m_comboCurrencyCode->insertSeparator(m_comboCurrencyCode->count());
+  currencyCodeList = m_locale->currency()->allCurrencyCodesList();
+  QStringList currencyNameList;
+  foreach ( const QString &currencyCode, currencyCodeList ) {
+    currencyNameList.append( i18nc( "@item currency name and currency code", "%1 (%2)",
+                                    m_locale->currency()->currencyCodeToName( currencyCode ), currencyCode ) );
+  }
+  currencyNameList.sort();
+  foreach ( const QString &name, currencyNameList ) {
+    m_comboCurrencyCode->addItem( name, QVariant( name.mid( name.length()-4, 3 ) ) );
+  }
+  m_comboCurrencyCode->setCurrentIndex( m_comboCurrencyCode->findData( QVariant( m_locale->currencyCode() ) ) );
+
+  //Create the list of Currency Symbols for the selected Currency Code
+  m_comboCurrencySymbol->clear();
+  QStringList currencySymbolList = m_locale->currency()->symbolList();
+  foreach ( const QString &currencySymbol, currencySymbolList ) {
+    m_comboCurrencySymbol->addItem( currencySymbol, QVariant( currencySymbol ) );
+  }
+  m_comboCurrencySymbol->setCurrentIndex( m_comboCurrencySymbol->findData( QVariant( m_locale->currencySymbol() ) ) );
+
   QList<QComboBox*> list;
   list.append(m_cmbMonPosMonSignPos);
   list.append(m_cmbMonNegMonSignPos);
@@ -260,7 +324,7 @@ void KLocaleConfigMoney::slotTranslate()
   QList<KLocale::DigitSet> digitSets = m_locale->allDigitSetsList();
   qSort(digitSets);
   m_cmbMonDigSet->clear();
-  foreach (KLocale::DigitSet ds, digitSets)
+  foreach (const KLocale::DigitSet &ds, digitSets)
   {
     m_cmbMonDigSet->addItem(m_locale->digitSetToName(ds, true));
   }
@@ -268,10 +332,18 @@ void KLocaleConfigMoney::slotTranslate()
 
   QString str;
 
-  str = ki18n( "Here you can enter your usual currency "
-               "symbol, e.g. $ or €." ).toString( m_locale );
-  m_labMonCurSym->setWhatsThis( str );
-  m_edMonCurSym->setWhatsThis( str );
+  str = ki18n( "Here you can choose the currency to be used "
+               "when displaying monetary values, e.g. United "
+               "States Dollar or Pound Sterling." ).toString( m_locale );
+  m_labelCurrencyCode->setWhatsThis( str );
+  m_comboCurrencyCode->setWhatsThis( str );
+
+  str = ki18n( "Here you can choose the currency symbol to be "
+               "displayed for your chosen currency, e.g. $, US$, "
+               "or USD." ).toString( m_locale );
+  m_labelCurrencySymbol->setWhatsThis( str );
+  m_comboCurrencySymbol->setWhatsThis( str );
+
   str = ki18n( "<p>Here you can define the decimal separator used "
                "to display monetary values.</p>"
                "<p>Note that the decimal separator used to "
@@ -288,12 +360,14 @@ void KLocaleConfigMoney::slotTranslate()
   m_labMonThoSep->setWhatsThis( str );
   m_edMonThoSep->setWhatsThis( str );
 
-  str = ki18n( "This determines the number of fract digits for "
-               "monetary values, i.e. the number of digits you "
-               "find <em>behind</em> the decimal separator. "
-               "Correct value is 2 for almost all people." ).toString( m_locale );
-  m_labMonFraDig->setWhatsThis( str );
-  m_inMonFraDig->setWhatsThis( str );
+  str = ki18n( "<p>Here you can set the number of decimal places "
+               "displayed for monetary values, i.e. the number "
+               "of digits <em>after</em> the decimal separator. "
+               "<p>Note that the decimal places used to "
+               "display other numbers has to be defined "
+               "separately (see the 'Numbers' tab).</p>" ).toString( m_locale );
+  m_labelMonetaryDecimalPlaces->setWhatsThis( str );
+  m_intMonetaryDecimalPlaces->setWhatsThis( str );
 
   str = ki18n( "If this option is checked, the currency sign "
                "will be prefixed (i.e. to the left of the "
