@@ -54,13 +54,29 @@ KNotifyConfig::KNotifyConfig( const QString & _appname, const ContextList & _con
 	configfile(retrieve_from_cache(_appname+QString::fromAscii( ".notifyrc" ))),
 	contexts(_contexts) , eventid(_eventid)
 {
-//	kDebug(300) << appname << " , " << eventid;
+//	kDebug() << appname << " , " << eventid;
 }
 
 KNotifyConfig::~KNotifyConfig()
 {
 }
 
+KNotifyConfig *KNotifyConfig::copy() const
+{
+	KNotifyConfig *config = new KNotifyConfig( appname, contexts, eventid );
+	config->title      = title;
+	config->text       = text;
+	config->image      = KNotifyImage( image.data() );
+	config->timeout    = timeout;
+	config->winId      = winId;
+	config->actions    = actions;
+	config->eventsfile = eventsfile;
+	config->configfile = configfile;
+	// appname, contexts, eventid already done in constructor
+
+	return config;
+}
+
 QString KNotifyConfig::readEntry( const QString & entry, bool path )
 {
 	QPair<QString , QString> context;
@@ -75,7 +91,7 @@ QString KNotifyConfig::readEntry( const QString & entry, bool path )
 				return p;
 		}
 	}
-//	kDebug(300) << entry << " not found in contexts ";
+//	kDebug() << entry << " not found in contexts ";
 	const QString group="Event/" + eventid ;
 	if(configfile->hasGroup( group ) )
 	{
@@ -84,7 +100,7 @@ QString KNotifyConfig::readEntry( const QString & entry, bool path )
 		if(!p.isNull())
 			return p;
 	}
-//	kDebug(300) << entry << " not found in config ";
+//	kDebug() << entry << " not found in config ";
 	if(eventsfile->hasGroup( group ) )
 	{
             KConfigGroup cg( eventsfile, group);
@@ -92,7 +108,7 @@ QString KNotifyConfig::readEntry( const QString & entry, bool path )
 		if(!p.isNull())
 			return p;
 	}
-//	kDebug(300) << entry << " not found !!! ";
+//	kDebug() << entry << " not found !!! ";
 
 	return QString();
 }
