@@ -1,5 +1,5 @@
 /*
- *   Copyright 2007-2008 Richard J. Moore <rich@kde.org>
+ *   Copyright 2007-2008,2010 Richard J. Moore <rich@kde.org>
  *   Copyright 2009 Aaron J. Seigo <aseigo@kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -21,13 +21,18 @@
 
 #include <iostream>
 
-#include <QScriptEngine>
 #include <QFile>
-#include <QUiLoader>
+#include <QKeyEvent>
 #include <QGraphicsLayout>
+#include <QGraphicsSceneHoverEvent>
+#include <QGraphicsSceneMouseEvent>
+#include <QGraphicsSceneWheelEvent>
 #include <QParallelAnimationGroup>
+#include <QPauseAnimation>
 #include <QSequentialAnimationGroup>
+#include <QUiLoader>
 #include <QWidget>
+#include <QScriptEngine>
 
 #include <KConfigGroup>
 #include <KDebug>
@@ -50,19 +55,24 @@
 #include <Plasma/Svg>
 #include <Plasma/VideoWidget>
 
+#include "appletauthorization.h"
+#include "scriptenv.h"
 #include "simplebindings/animationgroup.h"
 #include "simplebindings/dataengine.h"
+#include "simplebindings/dataenginereceiver.h"
 #include "simplebindings/i18n.h"
 #include "simplebindings/appletinterface.h"
 #include "simplebindings/bytearrayclass.h"
-#include "simplebindings/filedialogproxy.h"
 #include "simplebindings/variant.h"
 
 using namespace Plasma;
 
-
+Q_DECLARE_METATYPE(QKeyEvent*)
 Q_DECLARE_METATYPE(QPainter*)
 Q_DECLARE_METATYPE(QStyleOptionGraphicsItem*)
+Q_DECLARE_METATYPE(QGraphicsSceneHoverEvent *)
+Q_DECLARE_METATYPE(QGraphicsSceneMouseEvent *)
+Q_DECLARE_METATYPE(QGraphicsSceneWheelEvent *)
 
 QScriptValue constructColorClass(QScriptEngine *engine);
 QScriptValue constructEasingCurveClass(QScriptEngine *engine);
@@ -79,43 +89,23 @@ QScriptValue constructQPixmapClass(QScriptEngine *engine);
 QScriptValue constructQPointClass(QScriptEngine *engine);
 QScriptValue constructQRectFClass(QScriptEngine *engine);
 QScriptValue constructQSizeFClass(QScriptEngine *engine);
+QScriptValue constructQSizePolicyClass(QScriptEngine *engine);
 QScriptValue constructTimerClass(QScriptEngine *engine);
 void registerSimpleAppletMetaTypes(QScriptEngine *engine);
 
-class DummyService : public Service
-{
-public:
-    ServiceJob *createJob(const QString &operation, QMap<QString, QVariant> &parameters)
-    {
-        Q_UNUSED(operation)
-        Q_UNUSED(parameters)
-        return 0;
-    }
-};
-
-void registerEnums(QScriptEngine *engine, QScriptValue &scriptValue, const QMetaObject &meta)
-{
-    //manually create enum values. ugh
-    for (int i = 0; i < meta.enumeratorCount(); ++i) {
-        QMetaEnum e = meta.enumerator(i);
-        //kDebug() << e.name();
-        for (int i=0; i < e.keyCount(); ++i) {
-            //kDebug() << e.key(i) << e.value(i);
-            scriptValue.setProperty(e.key(i), QScriptValue(engine, e.value(i)));
-        }
-    }
-}
-
 KSharedPtr<UiLoader> SimpleJavaScriptApplet::s_widgetLoader;
 QHash<QString, Plasma::Animator::Animation> SimpleJavaScriptApplet::s_animationDefs;
 
 SimpleJavaScriptApplet::SimpleJavaScriptApplet(QObject *parent, const QVariantList &args)
     : Plasma::AppletScript(parent)
 {
-    Q_UNUSED(args)
+    Q_UNUSED(args);
 //    kDebug() << "Script applet launched, args" << applet()->startupArguments();
 
+    // TODO this will be set to the engine we get from QML
     m_engine = new QScriptEngine(this);
+    m_env = new ScriptEnv(this, m_engine);
+    connect(m_env, SIGNAL(reportError(ScriptEnv*,bool)), this, SLOT(engineReportsError(ScriptEnv*,bool)));
 }
 
 SimpleJavaScriptApplet::~SimpleJavaScriptApplet()
@@ -125,12 +115,24 @@ SimpleJavaScriptApplet::~SimpleJavaScriptApplet()
     }
 }
 
-void SimpleJavaScriptApplet::reportError(QScriptEngine *engine, bool fatal)
+void SimpleJavaScriptApplet::engineReportsError(ScriptEnv *engine, bool fatal)
 {
-    SimpleJavaScriptApplet *jsApplet = qobject_cast<SimpleJavaScriptApplet *>(engine->parent());
-    const QString failureMsg = i18n("Script failure on line %1:\n%2",
-                                    QString::number(engine->uncaughtExceptionLineNumber()),
-                                    engine->uncaughtException().toString());
+    reportError(engine, fatal);
+}
+
+void SimpleJavaScriptApplet::reportError(ScriptEnv *env, bool fatal)
+{
+    SimpleJavaScriptApplet *jsApplet = qobject_cast<SimpleJavaScriptApplet *>(env->parent());
+    AppletInterface *interface = extractAppletInterface(env->engine());
+    const QScriptValue error = env->engine()->uncaughtException();
+    QString file = error.property("fileName").toString();
+    if (interface) {
+        file.remove(interface->package()->path());
+    }
+
+    const QString failureMsg = i18n("Error in %1 on line %2.<br><br>%3",
+                                    file, error.property("lineNumber").toString(),
+                                    error.toString());
     if (jsApplet) {
         if (fatal) {
             jsApplet->setFailedToLaunch(true, failureMsg);
@@ -141,91 +143,80 @@ void SimpleJavaScriptApplet::reportError(QScriptEngine *engine, bool fatal)
         kDebug() << failureMsg;
     }
 
-    kDebug() << "Error in " << engine->uncaughtException().property("fileName").toString() << "on line" << engine->uncaughtExceptionLineNumber();
-    kDebug() << engine->uncaughtExceptionBacktrace();
+    kDebug() << env->engine()->uncaughtExceptionBacktrace();
 }
 
 void SimpleJavaScriptApplet::configChanged()
 {
-    QScriptValue fun = m_self.property("configChanged");
-    if (!fun.isFunction()) {
-        kDebug() << "Script: plasmoid.configChanged is not a function, " << fun.toString();
-        return;
-    }
-
-    QScriptContext *ctx = m_engine->pushContext();
-    ctx->setActivationObject(m_self);
-    //kDebug() << "calling plasmoid";
-    fun.call(m_self);
-    m_engine->popContext();
-
-    if (m_engine->hasUncaughtException()) {
-        reportError(m_engine);
+    if (m_eventListeners.contains("configchanged")) {
+        callEventListeners("configchanged");
+    } else {
+        callPlasmoidFunction("configChanged");
     }
 }
 
 void SimpleJavaScriptApplet::dataUpdated(const QString &name, const DataEngine::Data &data)
 {
-    QScriptValue fun = m_self.property("dataUpdated");
-    if (!fun.isFunction()) {
-        kDebug() << "Script: dataUpdated is not a function, " << fun.toString();
-        return;
-    }
-
     QScriptValueList args;
     args << m_engine->toScriptValue(name) << m_engine->toScriptValue(data);
+    if (m_eventListeners.contains("dataupdated")) {
+        callEventListeners("dataupdated");
+    } else {
+        callPlasmoidFunction("dataUpdated", args);
+    }
+}
 
-    QScriptContext *ctx = m_engine->pushContext();
-    ctx->setActivationObject(m_self);
-    fun.call(m_self, args);
-    m_engine->popContext();
+void SimpleJavaScriptApplet::extenderItemRestored(Plasma::ExtenderItem* item)
+{
+    QScriptValueList args;
+    args << m_engine->newQObject(item, QScriptEngine::AutoOwnership, QScriptEngine::PreferExistingWrapperObject);
+    if (m_eventListeners.contains("initextenderitem")) {
+        callEventListeners("initextenderitem");
+    } else {
+        callPlasmoidFunction("initExtenderItem", args);
+    }
+}
 
-    if (m_engine->hasUncaughtException()) {
-        reportError(m_engine);
+void SimpleJavaScriptApplet::activate()
+{
+    if (m_eventListeners.contains("activate")) {
+        callEventListeners("activate");
+    } else {
+        callPlasmoidFunction("activate");
+    }
+}
+
+void SimpleJavaScriptApplet::popupEvent(bool popped)
+{
+    QScriptValueList args;
+    args << popped;
+    if (m_eventListeners.contains("popupevent")) {
+        callEventListeners("popupevent");
+    } else {
+        callPlasmoidFunction("popupEvent", args);
     }
 }
 
 void SimpleJavaScriptApplet::executeAction(const QString &name)
 {
-    //callFunction("action_" + name);
-    QScriptValue fun = m_self.property("action_" + name);
-    if (fun.isFunction()) {
-        QScriptContext *ctx = m_engine->pushContext();
-        ctx->setActivationObject(m_self);
-        fun.call(m_self);
-        m_engine->popContext();
-
-        if (m_engine->hasUncaughtException()) {
-            reportError(m_engine);
-        }
+    if (m_eventListeners.contains("action_" + name)) {
+        callEventListeners("action_" + name);
+    } else {
+        callPlasmoidFunction("action_" + name);
     }
 }
 
 void SimpleJavaScriptApplet::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
 {
-    Q_UNUSED(option)
-    Q_UNUSED(contentsRect)
-
-    //kDebug() << "paintInterface() (c++)";
-    QScriptValue fun = m_self.property("paintInterface");
-    if (!fun.isFunction()) {
-        //kDebug() << "Script: paintInterface is not a function, " << fun.toString();
-        AppletScript::paintInterface(p, option, contentsRect);
-        return;
-    }
-
     QScriptValueList args;
     args << m_engine->toScriptValue(p);
     args << m_engine->toScriptValue(const_cast<QStyleOptionGraphicsItem*>(option));
     args << m_engine->toScriptValue(QRectF(contentsRect));
 
-    QScriptContext *ctx = m_engine->pushContext();
-    ctx->setActivationObject(m_self);
-    fun.call(m_self, args);
-    m_engine->popContext();
-
-    if (m_engine->hasUncaughtException()) {
-        reportError(m_engine);
+    if (m_eventListeners.contains("paintinterface")) {
+        callEventListeners("paintinterface", args);
+    } else {
+        callPlasmoidFunction("paintInterface", args);
     }
 }
 
@@ -234,18 +225,35 @@ QList<QAction*> SimpleJavaScriptApplet::contextualActions()
     return m_interface->contextualActions();
 }
 
-void SimpleJavaScriptApplet::callFunction(const QString &functionName, const QScriptValueList &args)
+void SimpleJavaScriptApplet::callPlasmoidFunction(const QString &functionName, const QScriptValueList &args)
 {
-    QScriptValue fun = m_self.property(functionName);
-    if (fun.isFunction()) {
-        QScriptContext *ctx = m_engine->pushContext();
-        ctx->setActivationObject(m_self);
-        fun.call(m_self, args);
-        m_engine->popContext();
+    QScriptValue func = m_self.property(functionName);
+    callFunction(func, args, m_self);
+}
 
-        if (m_engine->hasUncaughtException()) {
-            reportError(m_engine);
-        }
+void SimpleJavaScriptApplet::callFunction(QScriptValue &func, const QScriptValueList &args, const QScriptValue &activator)
+{
+    if (!func.isFunction()) {
+        return;
+    }
+
+    QScriptContext *ctx = m_engine->pushContext();
+    ctx->setActivationObject(activator);
+    func.call(activator, args);
+    m_engine->popContext();
+
+    if (m_engine->hasUncaughtException()) {
+        reportError(m_env);
+        m_engine->clearExceptions();
+    }
+}
+
+void SimpleJavaScriptApplet::callEventListeners(const QString &event, const QScriptValueList &args)
+{
+    QScriptValueList funcs = m_eventListeners.value(event);
+    QMutableListIterator<QScriptValue> it(funcs);
+    while (it.hasNext()) {
+        callFunction(it.next(), args);
     }
 }
 
@@ -254,53 +262,49 @@ void SimpleJavaScriptApplet::constraintsEvent(Plasma::Constraints constraints)
     QString functionName;
 
     if (constraints & Plasma::FormFactorConstraint) {
-        callFunction("formFactorChanged");
+        if (m_eventListeners.contains("formfactorchanged")) {
+            callEventListeners("formfactorchanged");
+        } else {
+            callPlasmoidFunction("formFactorChanged");
+        }
     }
 
     if (constraints & Plasma::LocationConstraint) {
-        callFunction("locationChanged");
+        if (m_eventListeners.contains("locationchanged")) {
+            callEventListeners("locationchanged");
+        } else {
+            callPlasmoidFunction("locationChanged");
+        }
     }
 
     if (constraints & Plasma::ContextConstraint) {
-        callFunction("currentActivityChanged");
+        if (m_eventListeners.contains("currentactivitychanged")) {
+            callEventListeners("currentactivitychanged");
+        } else {
+            callPlasmoidFunction("currentActivityChanged");
+        }
     }
 
     if (constraints & Plasma::SizeConstraint) {
-        callFunction("sizeChanged");
+        if (m_eventListeners.contains("sizechanged")) {
+            callEventListeners("sizechanged");
+        } else {
+            callPlasmoidFunction("sizeChanged");
+        }
     }
 
     if (constraints & Plasma::ImmutableConstraint) {
-        callFunction("immutabilityChanged");
+        if (m_eventListeners.contains("immutabilitychanged")) {
+            callEventListeners("immutabilitychanged");
+        } else {
+            callPlasmoidFunction("immutabilityChanged");
+        }
     }
 }
 
 bool SimpleJavaScriptApplet::include(const QString &path)
 {
-    QFile file(path);
-    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
-        kWarning() << i18n("Unable to load script file: %1", path);
-        return false;
-    }
-
-    QString script = file.readAll();
-    //kDebug() << "Script says" << script;
-
-    // change the context to the parent context so that the include is actually
-    // executed in the same context as the caller; seems to be what javascript
-    // coders expect :)
-    QScriptContext *ctx = m_engine->currentContext();
-    if (ctx && ctx->parentContext()) {
-        ctx->setActivationObject(ctx->parentContext()->activationObject());
-        ctx->setThisObject(ctx->parentContext()->thisObject());
-    }
-
-    m_engine->evaluate(script, path);
-    if (m_engine->hasUncaughtException()) {
-        reportError(m_engine, true);
-        return false;
-    }
-
-    return true;
+    return m_env->include(path);
 }
 
 void SimpleJavaScriptApplet::populateAnimationsHash()
@@ -321,210 +325,208 @@ bool SimpleJavaScriptApplet::init()
 {
     connect(applet(), SIGNAL(extenderItemRestored(Plasma::ExtenderItem*)),
             this, SLOT(extenderItemRestored(Plasma::ExtenderItem*)));
+    connect(applet(), SIGNAL(activate()),
+            this, SLOT(activate()));
     setupObjects();
 
-    if (!importExtensions()) {
+    AppletAuthorization auth(this);
+    if (!m_env->importExtensions(description(), m_self, auth)) {
         return false;
     }
 
     kDebug() << "ScriptName:" << applet()->name();
     kDebug() << "ScriptCategory:" << applet()->category();
-
-    return include(mainScript());
+    applet()->installEventFilter(this);
+    return m_env->include(mainScript());
 }
 
-void SimpleJavaScriptApplet::extenderItemRestored(Plasma::ExtenderItem* item)
+QScriptValue SimpleJavaScriptApplet::createKeyEventObject(QKeyEvent *event)
 {
-    QScriptValueList args;
-    args << m_engine->newQObject(item, QScriptEngine::AutoOwnership, QScriptEngine::PreferExistingWrapperObject);
-    callFunction("initExtenderItem", args);
+    QScriptValue v = m_env->engine()->newObject();
+    v.setProperty("count", event->count());
+    v.setProperty("key", event->key());
+    v.setProperty("modifiers", static_cast<int>(event->modifiers()));
+    v.setProperty("text", event->text());
+    return v;
 }
 
-QScriptValue SimpleJavaScriptApplet::runApplication(QScriptContext *context, QScriptEngine *engine)
+QScriptValue SimpleJavaScriptApplet::createHoverEventObject(QGraphicsSceneHoverEvent *event)
 {
-    Q_UNUSED(engine)
-    if (context->argumentCount() == 0) {
-        return false;
-    }
-
-    KUrl::List urls;
-    if (context->argumentCount() > 1) {
-        urls = qscriptvalue_cast<KUrl::List>(context->argument(1));
-    }
-
-    const QString app = context->argument(0).toString();
-
-    KService::Ptr service = KService::serviceByStorageId(app);
-    if (service) {
-        return KRun::run(*service, urls, 0);
-    }
-
-    const QString exec = KGlobal::dirs()->findExe(app);
-    if (!exec.isEmpty()) {
-        return KRun::run(exec, urls, 0);
-    }
-
-    return false;
+    QScriptEngine *engine = m_env->engine();
+    QScriptValue v = engine->newObject();
+    v.setProperty("pos", engine->toScriptValue<QPoint>(event->pos().toPoint()));
+    v.setProperty("scenePos", engine->toScriptValue<QPoint>(event->scenePos().toPoint()));
+    v.setProperty("screenPos", engine->toScriptValue<QPoint>(event->screenPos()));
+    v.setProperty("lastPos", engine->toScriptValue<QPoint>(event->lastPos().toPoint()));
+    v.setProperty("lastScenePos", engine->toScriptValue<QPoint>(event->lastScenePos().toPoint()));
+    v.setProperty("lastScreenPos", engine->toScriptValue<QPoint>(event->lastScreenPos()));
+    v.setProperty("modifiers", static_cast<int>(event->modifiers()));
+    return v;
 }
 
-QScriptValue SimpleJavaScriptApplet::runCommand(QScriptContext *context, QScriptEngine *engine)
+QScriptValue SimpleJavaScriptApplet::createMouseEventObject(QGraphicsSceneMouseEvent *event)
 {
-    Q_UNUSED(engine)
-    if (context->argumentCount() == 0) {
-        return false;
-    }
+    QScriptEngine *engine = m_env->engine();
+    QScriptValue v = engine->newObject();
+    v.setProperty("button", static_cast<int>(event->button()));
+    v.setProperty("buttons", static_cast<int>(event->buttons()));
+    v.setProperty("modifiers", static_cast<int>(event->modifiers()));
+    v.setProperty("pos", engine->toScriptValue<QPoint>(event->pos().toPoint()));
+    v.setProperty("scenePos", engine->toScriptValue<QPoint>(event->scenePos().toPoint()));
+    v.setProperty("screenPos", engine->toScriptValue<QPoint>(event->screenPos()));
+    v.setProperty("lastPos", engine->toScriptValue<QPoint>(event->lastPos().toPoint()));
+    v.setProperty("lastScenePos", engine->toScriptValue<QPoint>(event->lastScenePos().toPoint()));
+    v.setProperty("lastScreenPos", engine->toScriptValue<QPoint>(event->lastScreenPos()));
+    return v;
+}
 
-    const QString exec = KGlobal::dirs()->findExe(context->argument(0).toString());
-    if (!exec.isEmpty()) {
-        QString args;
-        if (context->argumentCount() > 1) {
-            const QStringList argList = qscriptvalue_cast<QStringList>(context->argument(1));
-            if (!argList.isEmpty()) {
-                args = ' ' + KShell::joinArgs(argList);
+QScriptValue SimpleJavaScriptApplet::createWheelEventObject(QGraphicsSceneWheelEvent *event)
+{
+    QScriptEngine *engine = m_env->engine();
+    QScriptValue v = engine->newObject();
+    v.setProperty("delta", event->delta());
+    v.setProperty("buttons", static_cast<int>(event->buttons()));
+    v.setProperty("modifiers", static_cast<int>(event->modifiers()));
+    v.setProperty("orientation", static_cast<int>(event->orientation()));
+    v.setProperty("pos", engine->toScriptValue<QPoint>(event->pos().toPoint()));
+    v.setProperty("scenePos", engine->toScriptValue<QPoint>(event->scenePos().toPoint()));
+    v.setProperty("screenPos", engine->toScriptValue<QPoint>(event->screenPos()));
+    return v;
+}
+
+bool SimpleJavaScriptApplet::eventFilter(QObject *watched, QEvent *event)
+{
+    switch (event->type()) {
+        case QEvent::KeyPress: {
+            if (m_eventListeners.contains("keypress")) {
+                QScriptValueList args;
+                args << createKeyEventObject(static_cast<QKeyEvent *>(event));
+                callEventListeners("keypress", args);
+                return true;
             }
         }
 
-        return KRun::runCommand(exec + args, 0);
-    }
-
-    return false;
-}
-
-QScriptValue SimpleJavaScriptApplet::openUrl(QScriptContext *context, QScriptEngine *engine)
-{
-    Q_UNUSED(engine)
-    if (context->argumentCount() == 0) {
-        return false;
-    }
-
-    QScriptValue v = context->argument(0);
-    KUrl url = v.isString() ? KUrl(v.toString()) : qscriptvalue_cast<KUrl>(v);
-    if (url.isValid()) {
-        return KRun::runUrl(url, KMimeType::findByUrl(url)->name(), 0);
-    }
-
-    return false;
-}
-
-QScriptValue SimpleJavaScriptApplet::getUrl(QScriptContext *context, QScriptEngine *engine)
-{
-    if (context->argumentCount() == 0) {
-        return engine->undefinedValue();
-    }
-
-    QScriptValue v = context->argument(0);
-    KUrl url = v.isString() ? KUrl(v.toString()) : qscriptvalue_cast<KUrl>(v);
-
-    if (!url.isValid()) {
-        return engine->undefinedValue();
-    }
-
-    AppletInterface *interface = extractAppletInterface(engine);
-    AllowedUrls allowed = interface ? interface->allowedUrls() : NoUrls;
-    if (url.isLocalFile()) {
-        if (!(allowed & LocalUrls)) {
-            return engine->undefinedValue();
+        case QEvent::KeyRelease: {
+            if (m_eventListeners.contains("keyrelease")) {
+                QScriptValueList args;
+                args << createKeyEventObject(static_cast<QKeyEvent *>(event));
+                callEventListeners("keyrelease", args);
+                return true;
+            }
         }
-    } else if (!(allowed & NetworkUrls) &&
-               !((allowed & HttpUrls) && (url.protocol() == "http" || url.protocol() == "https"))) {
-        return engine->undefinedValue();
+        break;
+
+        case QEvent::GraphicsSceneHoverEnter: {
+            if (m_eventListeners.contains("hoverenter")) {
+                QScriptValueList args;
+                args << createHoverEventObject(static_cast<QGraphicsSceneHoverEvent *>(event));
+                callEventListeners("hoverenter", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneHoverLeave: {
+            if (m_eventListeners.contains("hoverleave")) {
+                QScriptValueList args;
+                args << createHoverEventObject(static_cast<QGraphicsSceneHoverEvent *>(event));
+                callEventListeners("hoverleave", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneHoverMove: {
+            if (m_eventListeners.contains("hovermove")) {
+                QScriptValueList args;
+                args << createHoverEventObject(static_cast<QGraphicsSceneHoverEvent *>(event));
+                callEventListeners("hovermove", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneMousePress: {
+            if (m_eventListeners.contains("mousepress")) {
+                QScriptValueList args;
+                args << createMouseEventObject(static_cast<QGraphicsSceneMouseEvent *>(event));
+                callEventListeners("mousepress", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneMouseRelease: {
+            if (m_eventListeners.contains("mouserelease")) {
+                QScriptValueList args;
+                args << createMouseEventObject(static_cast<QGraphicsSceneMouseEvent *>(event));
+                callEventListeners("mouserelease", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneMouseMove: {
+            if (m_eventListeners.contains("mousemove")) {
+                QScriptValueList args;
+                args << createMouseEventObject(static_cast<QGraphicsSceneMouseEvent *>(event));
+                callEventListeners("mousemove", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneMouseDoubleClick: {
+            if (m_eventListeners.contains("mousedoubleclick")) {
+                QScriptValueList args;
+                args << createMouseEventObject(static_cast<QGraphicsSceneMouseEvent *>(event));
+                callEventListeners("mousedoubleclick", args);
+                return true;
+            }
+        }
+        break;
+
+        case QEvent::GraphicsSceneWheel: {
+            if (m_eventListeners.contains("wheel")) {
+                QScriptValueList args;
+                args << createWheelEventObject(static_cast<QGraphicsSceneWheelEvent *>(event));
+                callEventListeners("wheel", args);
+                return true;
+            }
+        }
+        break;
+
+        default:
+        break;
     }
 
-    KIO::Job *job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
-    return engine->newQObject(job);
+    return Plasma::AppletScript::eventFilter(watched, event);
 }
 
-void SimpleJavaScriptApplet::registerGetUrl()
+void SimpleJavaScriptApplet::addEventListener(const QString &event, const QScriptValue &func)
 {
-    QScriptValue get = m_self.property("getUrl");
-    if (!get.isValid()) {
-        m_self.setProperty("getUrl", m_engine->newFunction(SimpleJavaScriptApplet::getUrl));
+    if (func.isFunction()) {
+        m_eventListeners[event.toLower()].append(func);
     }
 }
 
-bool SimpleJavaScriptApplet::importBuiltinExtesion(const QString &extension)
+void SimpleJavaScriptApplet::removeEventListener(const QString &event, const QScriptValue &func)
 {
-    kDebug() << extension;
-    if ("filedialog" == extension) {
-        FileDialogProxy::registerWithRuntime(m_engine);
-        return true;
-    } else if ("launchapp" == extension) {
-        m_self.setProperty("runApplication", m_engine->newFunction(SimpleJavaScriptApplet::runApplication));
-        m_self.setProperty("runCommand", m_engine->newFunction(SimpleJavaScriptApplet::runCommand));
-        m_self.setProperty("openUrl", m_engine->newFunction(SimpleJavaScriptApplet::openUrl));
-        return true;
-    } else if ("http" == extension) {
-        m_interface->setAllowedUrls(m_interface->allowedUrls() | HttpUrls);
-        registerGetUrl();
-        return true;
-    } else if ("networkio" == extension) {
-        m_interface->setAllowedUrls(m_interface->allowedUrls() | HttpUrls | NetworkUrls);
-        registerGetUrl();
-        return true;
-    } else if ("localio" == extension) {
-        m_interface->setAllowedUrls(m_interface->allowedUrls() | LocalUrls);
-        registerGetUrl();
-        return true;
-    }
-
-    return false;
-}
-
-bool SimpleJavaScriptApplet::importExtensions()
-{
-    KPluginInfo info = description();
-    QStringList requiredExtensions = info.service()->property("X-Plasma-RequiredExtensions", QVariant::StringList).toStringList();
-    kDebug() << "required extensions are" << requiredExtensions;
-    foreach (const QString &ext, requiredExtensions) {
-        QString extension = ext.toLower();
-        if (m_extensions.contains(extension)) {
-            continue;
+    if (func.isFunction()) {
+        QScriptValueList funcs = m_eventListeners.value("mousepress");
+        QMutableListIterator<QScriptValue> it(funcs);//m_eventListeners.value("mousepress"));
+        while (it.hasNext()) {
+            if (it.next().equals(func)) {
+                it.remove();
+            }
         }
 
-        if (!applet()->hasAuthorization(extension)) {
-            setFailedToLaunch(true,
-                              i18n("Authorization for required extension '%1' was denied.",
-                                   extension));
-            return false;
-        }
-
-        if (!importBuiltinExtesion(extension)) {
-            m_engine->importExtension(extension);
-        }
-
-        if (m_engine->hasUncaughtException()) {
-            reportError(m_engine, true);
-            return false;
+        if (funcs.isEmpty()) {
+            m_eventListeners.remove(event.toLower());
         } else {
-            m_extensions << extension;
+            m_eventListeners.insert(event.toLower(), funcs);
         }
     }
-
-    QStringList optionalExtensions = info.service()->property("X-Plasma-OptionalExtensions", QVariant::StringList).toStringList();
-    kDebug() << "optional extensions are" << optionalExtensions;
-    foreach (const QString &ext, optionalExtensions) {
-        QString extension = ext.toLower();
-
-        if (m_extensions.contains(extension)) {
-            continue;
-        }
-
-        if (!applet()->hasAuthorization(extension)) {
-            continue;
-        }
-
-        if (!importBuiltinExtesion(extension)) {
-            m_engine->importExtension(extension);
-        }
-
-        if (m_engine->hasUncaughtException()) {
-            reportError(m_engine);
-        } else {
-            m_extensions << extension;
-        }
-    }
-
-    return true;
 }
 
 void SimpleJavaScriptApplet::setupObjects()
@@ -536,6 +538,8 @@ void SimpleJavaScriptApplet::setupObjects()
     global.setProperty("AnimationGroup", m_engine->newFunction(SimpleJavaScriptApplet::animationGroup));
     global.setProperty("ParallelAnimationGroup", m_engine->newFunction(SimpleJavaScriptApplet::parallelAnimationGroup));
 
+    // Bindings for data engine
+
     bindI18N(m_engine);
     global.setProperty("dataEngine", m_engine->newFunction(SimpleJavaScriptApplet::dataEngine));
     global.setProperty("service", m_engine->newFunction(SimpleJavaScriptApplet::service));
@@ -547,6 +551,10 @@ void SimpleJavaScriptApplet::setupObjects()
     m_self.setScope(global);
     global.setProperty("plasmoid", m_self);
 
+    if (isPopupApplet) {
+        connect(applet(), SIGNAL(popupEvent(bool)), this, SLOT(popupEvent(bool)));
+    }
+
     QScriptValue args = m_engine->newArray();
     int i = 0;
     foreach (const QVariant &arg, applet()->startupArguments()) {
@@ -555,18 +563,12 @@ void SimpleJavaScriptApplet::setupObjects()
     }
     global.setProperty("startupArguments", args);
 
-    registerEnums(m_engine, global, AppletInterface::staticMetaObject);
-
+    ScriptEnv::registerEnums(global, AppletInterface::staticMetaObject);
 
     // Add a global loadui method for ui files
     QScriptValue fun = m_engine->newFunction(SimpleJavaScriptApplet::loadui);
     global.setProperty("loadui", fun);
 
-    fun = m_engine->newFunction(SimpleJavaScriptApplet::print);
-    global.setProperty("print", fun);
-    fun = m_engine->newFunction(SimpleJavaScriptApplet::debug);
-    global.setProperty("debug", fun);
-
     // Work around bug in 4.3.0
     qMetaTypeId<QVariant>();
 
@@ -582,6 +584,7 @@ void SimpleJavaScriptApplet::setupObjects()
     global.setProperty("QPen", constructPenClass(m_engine));
     global.setProperty("QPixmap", constructQPixmapClass(m_engine));
     global.setProperty("QSizeF", constructQSizeFClass(m_engine));
+    global.setProperty("QSizePolicy", constructQSizePolicyClass(m_engine));
     global.setProperty("QPoint", constructQPointClass(m_engine));
     global.setProperty("LinearLayout", constructLinearLayoutClass(m_engine));
     global.setProperty("GridLayout", constructGridLayoutClass(m_engine));
@@ -606,7 +609,7 @@ void SimpleJavaScriptApplet::setupObjects()
 
 QSet<QString> SimpleJavaScriptApplet::loadedExtensions() const
 {
-    return m_extensions;
+    return m_env->loadedExtensions();
 }
 
 AppletInterface *SimpleJavaScriptApplet::extractAppletInterface(QScriptEngine *engine)
@@ -626,9 +629,12 @@ QScriptValue SimpleJavaScriptApplet::dataEngine(QScriptContext *context, QScript
         return context->throwError(i18n("Could not extract the Applet"));
     }
 
-    const QString dataEngine = context->argument(0).toString();
-    DataEngine *data = interface->dataEngine(dataEngine);
-    return engine->newQObject(data);
+    const QString dataEngineName = context->argument(0).toString();
+    DataEngine *dataEngine = interface->dataEngine(dataEngineName);
+    QScriptValue v = engine->newQObject(dataEngine, QScriptEngine::QtOwnership, QScriptEngine::PreferExistingWrapperObject);
+    v.setProperty("connectSource", engine->newFunction(DataEngineReceiver::connectSource));
+    v.setProperty("disconnectSource", engine->newFunction(DataEngineReceiver::disconnectSource));
+    return v;
 }
 
 QScriptValue SimpleJavaScriptApplet::service(QScriptContext *context, QScriptEngine *engine)
@@ -658,35 +664,53 @@ QScriptValue SimpleJavaScriptApplet::animation(QScriptContext *context, QScriptE
     }
 
     populateAnimationsHash();
-    QString animName = context->argument(0).toString().toLower();
+    QString name = context->argument(0).toString();
+    QString animName = name.toLower();
     const bool isPause = animName == "pause";
     const bool isProperty = animName == "property";
-    if (!isPause && !isProperty && !s_animationDefs.contains(animName)) {
-        return context->throwError(i18n("%1 is not a known animation type", animName));
-    }
 
     bool parentIsApplet = false;
     QGraphicsWidget *parent = extractParent(context, engine, 0, &parentIsApplet);
+    QAbstractAnimation *anim = 0;
+    Plasma::Animation *plasmaAnim = 0;
     if (isPause) {
-        QPauseAnimation *pause = new QPauseAnimation(parent);
-        return engine->newQObject(pause);
+        anim = new QPauseAnimation(parent);
     } else if (isProperty) {
-        QPropertyAnimation *propertyAnim = new QPropertyAnimation(parent);
-        return engine->newQObject(propertyAnim);
+        anim = new QPropertyAnimation(parent);
+    } else if (s_animationDefs.contains(animName)) {
+        plasmaAnim = Plasma::Animator::create(s_animationDefs.value(animName), parent);
     } else {
-        Plasma::Animation *anim = Plasma::Animator::create(s_animationDefs.value(animName), parent);
-        if (anim) {
-            if (!parentIsApplet) {
-                anim->setTargetWidget(parent);
-            }
+        SimpleJavaScriptApplet *jsApplet = qobject_cast<SimpleJavaScriptApplet *>(engine->parent());
+        if (jsApplet) {
+            //kDebug() << "trying to load it from the package";
+            plasmaAnim = jsApplet->loadAnimationFromPackage(name, parent);
+        }
 
-            QScriptValue value = engine->newQObject(anim);
-            registerEnums(engine, value, *anim->metaObject());
-            return value;
+        if (!plasmaAnim) {
+            plasmaAnim = Plasma::Animator::create(animName, parent);
         }
     }
 
-    return context->throwError(i18n("%1 is not a known animation type", animName));
+    if (plasmaAnim) {
+        if (!parentIsApplet) {
+            plasmaAnim->setTargetWidget(parent);
+        }
+        anim = plasmaAnim;
+    }
+
+    if (anim) {
+        QScriptValue value = engine->newQObject(anim);
+        ScriptEnv::registerEnums(value, *anim->metaObject());
+        return value;
+    }
+
+    context->throwError(i18n("%1 is not a known animation type", animName));
+
+    ScriptEnv *env = ScriptEnv::findScriptEnv(engine);
+    if (env) {
+        env->checkForErrors(false);
+    }
+    return engine->undefinedValue();
 }
 
 QScriptValue SimpleJavaScriptApplet::animationGroup(QScriptContext *context, QScriptEngine *engine)
@@ -763,7 +787,7 @@ QScriptValue SimpleJavaScriptApplet::newPlasmaSvg(QScriptContext *context, QScri
     Svg *svg = new Svg(parent);
     svg->setImagePath(parentedToApplet ? findSvg(engine, filename) : filename);
     QScriptValue fun = engine->newQObject(svg);
-    registerEnums(engine, fun, *svg->metaObject());
+    ScriptEnv::registerEnums(fun, *svg->metaObject());
     return fun;
 }
 
@@ -781,8 +805,7 @@ QScriptValue SimpleJavaScriptApplet::newPlasmaFrameSvg(QScriptContext *context,
     frameSvg->setImagePath(parentedToApplet ? filename : findSvg(engine, filename));
 
     QScriptValue fun = engine->newQObject(frameSvg);
-    // FIXME: why is this necessary when it is clearly declared in FrameSvg's moc?
-    registerEnums(engine, fun, *frameSvg->metaObject());
+    ScriptEnv::registerEnums(fun, *frameSvg->metaObject());
     return fun;
 }
 
@@ -804,7 +827,7 @@ QScriptValue SimpleJavaScriptApplet::newPlasmaExtenderItem(QScriptContext *conte
 
     Plasma::ExtenderItem *extenderItem = new Plasma::ExtenderItem(extender);
     QScriptValue fun = engine->newQObject(extenderItem);
-    registerEnums(engine, fun, *extenderItem->metaObject());
+    ScriptEnv::registerEnums(fun, *extenderItem->metaObject());
     return fun;
 }
 
@@ -882,39 +905,10 @@ QScriptValue SimpleJavaScriptApplet::createWidget(QScriptContext *context, QScri
     fun.setProperty("adjustSize", engine->newFunction(widgetAdjustSize));
 
     //register enums will be accessed for instance as frame.Sunken for Frame shadow...
-    registerEnums(engine, fun, *w->metaObject());
+    ScriptEnv::registerEnums(fun, *w->metaObject());
     return fun;
 }
 
-QScriptValue SimpleJavaScriptApplet::notSupported(QScriptContext *context, QScriptEngine *engine)
-{
-    Q_UNUSED(engine)
-    QString message = context->callee().property("message").toString();
-    return context->throwError(i18n("This operation was not supported, %1", message) );
-}
-
-
-QScriptValue SimpleJavaScriptApplet::print(QScriptContext *context, QScriptEngine *engine)
-{
-    if (context->argumentCount() != 1) {
-        return context->throwError(i18n("print() takes one argument"));
-    }
-
-    //TODO: a GUI console? :)
-    std::cout << context->argument(0).toString().toLocal8Bit().constData() << std::endl;
-    return engine->undefinedValue();
-}
-
-QScriptValue SimpleJavaScriptApplet::debug(QScriptContext *context, QScriptEngine *engine)
-{
-    if (context->argumentCount() < 1) {
-        return engine->undefinedValue();
-    }
-
-    kDebug() << context->argument(0).toString();
-    return engine->undefinedValue();
-}
-
 void SimpleJavaScriptApplet::collectGarbage()
 {
     m_engine->collectGarbage();
@@ -929,4 +923,4 @@ QScriptValue SimpleJavaScriptApplet::variantToScriptValue(QVariant var)
 }
 
 K_EXPORT_PLASMA_APPLETSCRIPTENGINE(qscriptapplet, SimpleJavaScriptApplet)
-#include "simplejavascriptapplet.moc"
+
